@@ -1,6 +1,6 @@
 Lexis <-
 function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
-         merge=TRUE, states, tol=.Machine$double.eps^0.5,
+         merge=TRUE, states, notes=TRUE, tol=.Machine$double.eps^0.5,
          keep.dropped=FALSE )
 {
   nmissing <- missing(entry) + missing(exit) + missing(duration)
@@ -42,7 +42,7 @@ function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
     if( is.logical( exit.status ) )
         {
         entry.status <- FALSE
-        cat("NOTE: entry.status has been set to FALSE for all.\n" )
+        if( notes ) cat("NOTE: entry.status has been set to FALSE for all.\n" )
         }
     if( is.character( exit.status ) )
         {
@@ -54,13 +54,13 @@ function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
                                      length(exit.status)),
                                 levels=levels(exit.status),
                                 labels=levels(exit.status) )
-        cat("NOTE: entry.status has been set to",
+        if( notes ) cat("NOTE: entry.status has been set to",
             paste( '"', levels(exit.status)[1], '"', sep='' ),
             "for all.\n" )
         }
     if( is.numeric( exit.status ) )
         {
-        entry.status <- rep( 0, length( exit.status ) )
+        if( notes ) entry.status <- rep( 0, length( exit.status ) )
         cat("NOTE: entry.status has been set to 0 for all.\n" )
         }
     }
@@ -76,7 +76,7 @@ function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
               all.levels = union(levels(entry.status),levels(exit.status))
               entry.status <- factor( entry.status, levels=all.levels )
                exit.status <- factor(  exit.status, levels=all.levels )
-            cat("Incompatible factor levels in entry.status and exit.status:\n",
+          if( notes ) cat("Incompatible factor levels in entry.status and exit.status:\n",
                 "both lex.Cst and lex.Xst now have levels:\n", all.levels, "\n")
           }
       }
@@ -101,7 +101,7 @@ function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
         {
         entry <- exit
         entry[[1]] <- 0*entry[[1]]
-        cat( "NOTE: entry is assumed to be 0 on the",names(exit),"timescale.\n")
+        if( notes ) cat( "NOTE: entry is assumed to be 0 on the",names(exit),"timescale.\n")
         }
       }
     else
@@ -112,7 +112,7 @@ function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
         {
         entry <- duration
         entry[[1]] <- 0*entry[[1]]
-        cat( "NOTE: entry is assumed to be 0 on the",names(duration),"timescale.\n")
+        if( notes ) cat( "NOTE: entry is assumed to be 0 on the",names(duration),"timescale.\n")
         }
       }
     else
@@ -229,12 +229,12 @@ function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
     }
     lex <- cbind(lex, data)
   }
-
+  
   ## Drop rows with short or negative duration for consistency with splitLexis
   short.dur <- lex$lex.dur <= tol
   if ( any(short.dur) ) {
-      warning("Dropping ", sum(short.dur),
-              " rows with duration of follow up < tol\n",
+      if( notes ) cat("NOTE: Dropping ", sum(short.dur),
+           " rows with duration of follow up < tol\n",
       if( keep.dropped ) "  The dropped rows are in the attribute 'dropped'\n",
       if( keep.dropped ) "  To see them type attr(Obj,'dropped'),\n",
       if( keep.dropped ) "  to get rid of them type: attr(Obj,'dropped') <- NULL\n",
@@ -702,11 +702,46 @@ status <- function(x, at="exit", by.id = FALSE)
   res
 }
 
+transient <-
+function( x )
+{
+if( !is.Lexis(x) ) stop( "Not a Lexis object" )
+tc <- tapply( x$lex.dur, x$lex.Cst, sum )
+nt <- names( tc[tc>0] )
+nt[!is.na(nt)]
+}    
+
+absorbing <-
+function( x )
+{
+if( !is.Lexis(x) ) stop( "Not a Lexis object" )
+tc <- table( x$lex.Xst )
+setdiff( names( tc[tc>0] ), transient(x) )
+}    
+
+updn <-
+function( x, tt, states )
+{
+if( any(is.na(match(states,levels(x)))) )
+  stop( "'states' must be among states: ", paste(levels(x),sep=',') )
+tt <- tt[,states,drop=FALSE]
+setdiff( rownames( tt[apply(tt,1,sum)>0,,drop=FALSE] ), states )
+}
+before <- preceding <- function( x, states ) updn( x, table(x$lex.Cst,x$lex.Xst), states )
+after <- succeeding <- function( x, states ) updn( x, table(x$lex.Xst,x$lex.Cst), states )
+
 timeScales <- function(x)
 {
   return (attr(x,"time.scales"))
 }
 
+timeSince <- function(x)
+{
+        tt  <- attr(x,"time.since")
+  names(tt) <- attr(x,"time.scales")
+  return( tt )
+}
+
 timeBand <- function(lex, time.scale, type="integer")
 {
   time.scale <- check.time.scale(lex, time.scale)[1]
@@ -761,5 +796,14 @@ transform.Lexis <- function(`_data`, ... )
     y
 }
 
-# order.Lexis <- function( x ) order( x$lex.id, lex[,timeScales(x)[1]] )
-#  sort.Lexis <- function( x, ... ) x[order.Lexis(x,...),]
+# Two utility functions used to sort a Lexis object by (lex.id,time)    
+order.Lexis <-
+ orderLexis <- function( x )
+               {
+               # Some time scales may contain missing values
+               # So find (any) one that does not so we can sort on it 
+       N.NA <- apply(x[,timeScales(x)], 2, function(x) sum(is.na(x)))
+  a.full.ts <- timeScales(x)[which(N.NA == 0)[1]]
+               order(x$lex.id, x[,a.full.ts])
+               }
+  sortLexis <- function( x ) x[orderLexis(x),]
